@@ -67,6 +67,8 @@ class _LazyModule(_ModuleType):
 from bash.
 """
 # developer note: this file should not perform any action on import.
+#                 This file comes from https://github.com/xonsh/py-bash-completion
+#                 and should be edited there!
 os = _LazyModule.load('os', 'os')
 re = _LazyModule.load('re', 're')
 sys = _LazyModule.load('sys', 'sys')
@@ -301,7 +303,18 @@ COMP_COUNT={end}
 COMP_CWORD={n}
 $_func {cmd} {prefix} {prev}
 
-for ((i=0;i<${{#COMPREPLY[*]}};i++)) do echo ${{COMPREPLY[i]}}; done
+# print out completions, right-stripped if they contain no internal spaces
+shopt -s extglob
+for ((i=0;i<${{#COMPREPLY[*]}};i++))
+do
+    no_spaces="${{COMPREPLY[i]//[[:space:]]}}"
+    no_trailing_spaces="${{COMPREPLY[i]%%+([[:space:]])}}"
+    if [[ "$no_spaces" == "$no_trailing_spaces" ]]; then
+        echo "$no_trailing_spaces"
+    else
+        echo "${{COMPREPLY[i]}}"
+    fi
+done
 """
 
 
@@ -338,8 +351,8 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
 
     Returns
     -------
-    rtn : list of str
-        Possible completions of prefix, sorted alphabetically.
+    rtn : set of str
+        Possible completions of prefix
     lprefix : int
         Length of the prefix to be replaced in the completion.
     """
@@ -405,6 +418,63 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
 
     return out, len(prefix) - strip_len
 
+
+def bash_complete_line(line, return_line=True, **kwargs):
+    """Provides the completion from the end of the line.
+
+    Parameters
+    ----------
+    line : str
+        Line to complete
+    return_line : bool, optional
+        If true (default), will return the entire line, with the completion added.
+        If false, this will instead return the strings to append to the original line.
+    kwargs : optional
+        All other keyword arguments are passed to the bash_completions() function.
+
+    Returns
+    -------
+    rtn : set of str
+        Possible completions of prefix
+    """
+    # set up for completing from the end of the line
+    split = line.split()
+    if len(split) > 1 and not line.endswith(' '):
+        prefix = split[-1]
+        begidx = len(line.rsplit(prefix)[0])
+    else:
+        prefix = ''
+        begidx = len(line)
+    endidx = len(line)
+    # get completions
+    out, lprefix = bash_completions(prefix, line, begidx, endidx, **kwargs)
+    # reformat output
+    if return_line:
+        preline = line[:-lprefix]
+        rtn = {preline + o for o in out}
+    else:
+        rtn = {o[lprefix:] for o in out}
+    return rtn
+
+
+def _bc_main(args=None):
+    """Runs complete_line() and prints the output."""
+    from argparse import ArgumentParser
+    p = ArgumentParser('bash_completions')
+    p.add_argument('--return-line', action='store_true', dest='return_line', default=True,
+                   help='will return the entire line, with the completion added')
+    p.add_argument('--no-return-line', action='store_false', dest='return_line',
+                   help='will instead return the strings to append to the original line')
+    p.add_argument('line', help='line to complete')
+    ns = p.parse_args(args=args)
+    out = bash_complete_line(ns.line, return_line=ns.return_line)
+    for o in sorted(out):
+        print(o)
+
+
+if __name__ == '__main__':
+    _bc_main()
+
 #
 # completer
 #
